# Testing Framework

- The entire point of testing is to cover your own ass.
- In order to do that, you need to eliminiate every possible vector for error.
- This means being verbose in both how you define and perform tests.
- Define the way that a user would interact and potentially find errors.

## Your Power Future Example

- Goal of this project is to upgrade from one framework version to another.
- For this project, we know that we are testing a website, a CMS and setup.
- The website can be tested through user interaction and confirmation.
- The same is true for the CMS.
- The setup does the same albiet in a developer focused mindset.
- There is the additional component of checking the version changes. 
- That is an additonal vector of tests.
- Note: I have not considered end to end or integration tests yet.

### High Level Test Plan

- [ ] Check the CMS.
- [ ] Check the website.
- [ ] Check the setup.
- [ ] Check the version changes.

- From here, we can break down those points into more and more verbose tests.
- For example, if we open the CMS then everything that you see needs a test.

### High Level Break Down

- In this case, the CMS and the website are just content/function tests.
- Most of the work will come in the setup and version changes.

- [ ] CMS
	- [ ] Everything in the CMS
- [ ] Website
	- [ ] Everything on the website.
- [ ] Setup
	- [ ] Can we bring in this version of the project from nothing.
	- [ ] Can the project be ran and built without errors.
	- [ ] Does the project make reference to everything it needs.
	- [ ] Is there documentation for this setup process.
- [ ] Version Changes
	- [ ] Are there any breaking changes between the versions.
	- [ ] Are there any non-breaking changes that need to be made.
	- [ ] Has every change been considered or checked.

- It is at this point where we get to most of the context dependent testing.

### Documenting the tests

- Once again, the documentation will be context dependent.


#### Setup Documentation

- The setup is more a checklist than a set of actions.

| id | Test                         | Test Explanation                       | Test Notes                     | Test Result |
| -- | ------------                 | ------------------                     | ----------                     | ----------- |
| 0  | Dependencies are up to date  | Check dependencies against latest      | Deps are all good apart from X | SUCCESS     |
| 1  | Config files are up to date  | List out the configs to check          | Something about the configs    | N/A         |
| 2  | Can build from repo checkout | Can the user checkout and build from 0 | Error in doing whatever        | FAIL        |

#### CMS + Website

- The CMS and website are close in testing methods, so they're shown together.

| id | Test                   | Test Goal                       | Test Action      | Test Notes  | Test Result |
| -- | ------------           | ------------------              | ------------     | ----------  | ----------- |
| 0  | Site Header Search     | Search should work like so      | Use the search   | All good    | SUCCESS     |
| 1  | Site document download | User can download the documents | Attempt download | This is odd | N/A         |
| 2  | Site styling check     | All the styling is correct      | Check styling    | All wrong   | FAIL        |

- The idea here is that we have a test, that should result in a goal and that goal is successful due to the action.
- E.g. Test the site header, it should search like this, you test it by trying to search.

#### Version Change Documentation

| id | Version      | Ticket  | Ticket Explanation                    | Test Notes                    | Test Result |
| -- | ------------ | ------  | ------------------                    | ----------                    | ----------- |
| 0  | 1.3.1        | TK-7127 | Something has gone wrong              | It goes wrong but that's okay | SUCCESS     |
| 1  | 1.3.2        | TK-1157 | Another thing has gone wrong          | This one is intentional       | N/A         |
| 2  | 1.3.0        | TK-2632 | Make sure that the user can't do this | Not relevant.                 | FAIL        |

- 0: Here are some extra notes for test 0.

- Here it's a version upgrade specific thing, but the idea is sound.
- We have a ticket to check, what does the ticket do for what version.


#### Overall Testing Table

- In generating the above tables, it is clear that only 1 is really needed:

| id | Context | Test                         | Test Goal                       | Test Action      | Test Notes  | Test Result |
| -- | ------- | ------------                 | ------------------              | ------------     | ----------  | ----------- |
| 0  | Website | Site Header Search           | Search should work like so      | Use the search   | All good    | SUCCESS     |
| 1  | CMS     | Site document download       | User can download the documents | Attempt download | This is odd | N/A         |
| 2  | Website | Site styling check           | All the styling is correct      | Check styling    | All wrong   | FAIL        |
| 3  | Setup   | Can build from repo checkout | I should be able to do this     | Some commands    | Fails on X  | N/A         |

- Here I can specify context, reducing the need for multiple tables.
- Really, the version changes could also be part of this:

| id | Context         | Test         | Test Goal                 | Test Action        | Test Notes          | Test Result |
| -- | -------         | ------------ | ------------------        | ------------       | ----------          | ----------- |
| 0  | Version Upgrade | TK-7127      | Ticket is about something | Test the something | 1.3.0, Not relevant | SUCCESS     |

- Here we can just add to the general notes as to which version.
- This along with specificity on the context will lead to verboseness.

- 15:09:16 2025-04-15 Conversation with GB:
	- https://enigma-interactive.atlassian.net/wiki/spaces/WPD/pages/34082425/WPD+Core+Upgrade+-+Your+Power+Future+-+2.2.17.8+-+2.3.40.2
	- Perfect example of what's actually needed.
	- As long as Website, CMS and Setup are good to go, then the version check is a nice to have.
